[ORG 0x500]
    push ds                         ;Copy DS to CS
    pop cs




;*****   Check user    *****
%include "include\user.asm"

 CmdPrompt:
;Note to self: Anything put here will be execute on every command


   mov si,leftarr                   ;Point SI to prompt string
   mov ah,00h                      ;EasyOS Print String Function
   int 0x20                        ;Do it!
   mov si,user
   mov ah,00h                      ;EasyOS Print String Function
   int 0x20                        ;Do it!
   mov si,rightar
   mov ah,00h                      ;EasyOS Print String Function
   int 0x20                        ;Do it!

 CmdLoop:
    mov ah,0                        ;BIOS INT 16h Read Keyboard Code
    int 0x16                        
    cmp al,0x0D                     ;Did the user press enter?
    jne Not_CR                      ;No.
    jmp ProcessCommand              ;Yes. Check if the command is known
 Not_CR:
    cmp al,0x20                     ;Did the user press space
    jne Not_SPACE                   ;No.
    cmp byte [CURSORPOS],0          ;Is the cursor at position 0
    jne StillUpdate                 ;No. Continue to update buffer
    mov ax,0xE20                    ;Print a space
    int 0x10
    jmp CmdLoop                     ;Jump back and wait for another keypress
 StillUpdate:
    jmp Not_BKSP                    ;Skip checks for other keys.
 Not_SPACE:
    cmp al,0x08                     ;Did the user press backspace
    jne Not_BKSP                    ;No.
    cmp byte [CURSORPOS],0          ;Is the cursor at the first position
    je CmdLoop                      ;Jump back and wait for another keypress
    dec byte [CURSORPOS]            ;Put cursor back a position
    mov ax,0xE08                    ;Do the backspace
    int 0x10
    mov ax,0xE00
    int 0x10
    mov ax,0xE08
    int 0x10
    jmp CmdLoop                     ;Jump back and wait for another keypress
 Not_BKSP:
    xor bx,bx                       ;Clear BX
    mov bl,[CURSORPOS]              ;Put the cursor position in bl
    cmp bl,70                       ;Is it at the MAX?
    je CmdLoop                      ;Yep. Don't add to buffer or echo to screen
    mov [COMMANDBUFFER+bx],al       ;Add the key to buffer.
    inc byte [CURSORPOS]            ;Increment the cursor
    mov ah,0xE                      ;Echo the character to the screen
    int 0x10
    jmp CmdLoop                     ;Jump back and wait for another keypress
 ProcessCommand:
    cmp byte [CURSORPOS],0          ;Have there been any keys pressed?
    je NoCommand                    ;No. Then don't check for a valid command
    mov si,COMMANDLOOKUP            ;Point SI to the list of valid commands
    xor cx,cx                       ;Clear CX
    xor ah,ah                       ;Clear AH
    xor bx,bx                       ;Clear BX
 _search:
    mov di,COMMANDBUFFER            ;Point DI to the command buffer
    lodsb                           ;Get amount of letters in next command
    cmp al,0x00                     ;Is it the end of the list?
    je UnknownCommand               ;Yes. Report that there was no valid command
    mov cl,al                       ;Put the amount of characters in CL
    push si                         ;Save SI on stack
    rep cmpsb                       ;Compare buffer with list
    je DetermineAddr                ;Command matches. Get address of command
    pop si                          ;Get SI off stack
    add si,ax                       ;Increment position in command list
    inc bl                          ;Increment command number
    jmp _search                     ;Check for next command in list
 NoCommand:
    mov di,COMMANDBUFFER            ;Point DI to command buffer
    mov cx,70                       ;Clear 70 bytes
    mov al,0                        ;With value 0
    rep stosb                       ;NOW!!!
    mov byte [CURSORPOS],0          ;Set cursor position to 0
    jmp CmdPrompt                   ;Display prompt
 UnknownCommand:
    mov si,UNKCOMMAND
    mov ah,00h       
    int 0x20         
    jmp NoCommand                   
 DetermineAddr:
    pop si                          ;Pop junk off stack
    mov si,di                       ;Move position in buffer to si
    lodsb                           ;Get next character
    cmp al,0x00                     ;Is it the end of the command
    je CommandOK                    ;Yes. Continue
    jmp UnknownCommand              ;No. Could be a longer command? SEE NOTE
;THIS CHECK HERE MAKES SURE THAT THE COMMAND TYPED IN ISN'T JUST A PART OF
;A BIGGER COMMAND LIKE:
;       dir                         
;       directory
;The user could have typed directory instead of dir and they could be
;different commands.
 CommandOK:
    mov byte [CURSORPOS],0          ;Set cursor position to 0
    xor ax,ax                       ;Clear AX for multiplication
    mov al,bl                       ;Copy command number to al
    mov cl,2                        ;Multiply by 2
    mul cl                          ;NOW!!
    mov bx,COMMANDADDRLOOKUP        ;Point BX to the address lookup table
    add bx,ax                       ;Increment it to start of requested command
    mov si,[bx]                     ;Put address in SI
    jmp si                          ;Jump to command code
UNKCOMMAND db 13,10,'Unknown Crash command.',0
                   

COMMANDLOOKUP     db 7,'restart',5,'clear',4,'help',6,'access',6,'logout',0
COMMANDADDRLOOKUP dw restartCMD,clearCMD,helpCMD,accessCMD,logoutCMD

;The Command Buffer is where every keypress is stored for checking if
;a valid command was typed.

COMMANDBUFFER:    times 100 db 0
CURSORPOS         db 0         

restartCMD:
jmp 0xffff:0x0000

clearCMD:
mov ah,03h
int 20h
jmp NoCommand


helpCMD:
mov si,helper
mov ah,00h
int 20h
jmp NoCommand

accessCMD:
mov si,accessdb
mov ah,00h
int 0x20
mov dx,[access]
mov si,dx
mov ah,00h
int 0x20
jmp NoCommand

logoutCMD:
jmp UserPrompt




helper db 13,10,'Crash shell commands:'
 db 13,10,'clear -Clear screen'
 db 13,10,'restart -Restart'
 db 13,10,'help -This'
 db 13,10,'access -Display access level'
 db 13,10,'logout -Return to log-in screen',0
levelone db '1',0
levelten db '10',0
tester db       13,10,'test'
newline db 13,10,0
leftarr db 13,10,'<',0
rightar db '>',0
accessdb db 13,10,'Access level [1 being lowest, 10 being highest] : ',0